(function() {
  var Command, cli, clr, fs, getLocalProject, getVersion, path, _,
    __slice = [].slice,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  fs = require('fs');

  path = require('path');

  Command = require('commander').Command;

  clr = require('ansi-color').set;

  _ = require('underscore');

  exports.version = getVersion = function() {
    var pkg;
    fs = require('fs');
    path = require('path');
    pkg = JSON.parse(fs.readFileSync(path.resolve(__dirname, '../package.json')));
    return pkg.version;
  };

  getLocalProject = function() {
    return new (require('./index')).Project;
  };

  _.extend(exports, cli = {
    finished: function(action, target) {
      return console.log(clr(action, 'blue'), target);
    },
    debug: function() {
      var msgs;
      msgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return console.log.apply(console, [clr('DEBUG', 'red')].concat(__slice.call(msgs)));
    },
    warning: function() {
      var msgs;
      msgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return console.log.apply(console, [clr('Warning', 'yellow')].concat(__slice.call(msgs)));
    },
    info: function() {
      var msgs;
      msgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return console.log.apply(console, [clr('Info', 'green')].concat(__slice.call(msgs)));
    },
    error: function() {
      var msgs;
      msgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return console.log.apply(console, [clr('Error', 'red')].concat(__slice.call(msgs)));
    }
  });

  exports.run = function(argv) {
    var program;
    (program = new Command).version(getVersion()).option('-t, --template <template>', 'Specify the project template to use (default to \'lesscoffee\') when initializing.');
    program.command('init').description(" Initialize " + (clr('Brewer', 'green')) + " in the current directory").action(function() {
      var content, templ, template, _ref;
      templ = (_ref = program.template) != null ? _ref : 'lesscoffee';
      template = path.join(__dirname, '..', 'templates', "" + templ + ".coffee");
      content = fs.readFileSync(template, 'utf-8');
      cli.info('Writing Brewfile');
      fs.writeFileSync('Brewfile', content, 'utf-8');
      cli.info('Making initial folder structure');
      return getLocalProject().prepare();
    });
    program.command('watch').description(" Watch for modifications in source and configuration files, \nautomatically re-making when they occur.").action(function() {
      getLocalProject().watch();
      return cli.info('Watching project', process.cwd());
    });
    program.command('clean [packages]*').description(" Delete bundles and compiled files from the given packages (or all)").action(function(pkgs) {
      var names, packages, pkg, project, _i, _len, _results;
      if (pkgs == null) {
        pkgs = 'all';
      }
      project = getLocalProject();
      if (pkgs === 'all') {
        packages = project;
      } else {
        names = pkgs.split(',');
        packages = (function() {
          var _i, _len, _ref, _results;
          _results = [];
          for (_i = 0, _len = project.length; _i < _len; _i++) {
            pkg = project[_i];
            if (_ref = pkg.name, __indexOf.call(names, _ref) >= 0) {
              _results.push(pkg);
            }
          }
          return _results;
        })();
      }
      _results = [];
      for (_i = 0, _len = packages.length; _i < _len; _i++) {
        pkg = packages[_i];
        _results.push((function(pkg) {
          return pkg.clean();
        })(pkg));
      }
      return _results;
    });
    program.command('install').description(" Install missing modules required to manage the current project").action(function() {
      var project;
      project = getLocalProject();
      if (project.missingModules().length > 0) {
        return project.installMissingModules();
      } else {
        return cli.info('No modules are missing.');
      }
    });
    program.command('*').description(" Aggregate bundles from the given packages (or all)").action(function() {
      var packages, pkg, pkgs, project, _i, _len, _results;
      pkgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (pkgs == null) {
        pkgs = 'all';
      }
      project = getLocalProject();
      if (pkgs[0] === 'all') {
        packages = project;
      } else {
        packages = (function() {
          var _i, _len, _ref, _results;
          _results = [];
          for (_i = 0, _len = project.length; _i < _len; _i++) {
            pkg = project[_i];
            if (_ref = pkg.name, __indexOf.call(pkgs, _ref) >= 0) {
              _results.push(pkg);
            }
          }
          return _results;
        })();
      }
      _results = [];
      for (_i = 0, _len = packages.length; _i < _len; _i++) {
        pkg = packages[_i];
        _results.push((function(pkg) {
          return pkg.actualize(function() {
            return cli.info("Finished making " + (clr(pkg.name, 'underline')) + " package");
          });
        })(pkg));
      }
      return _results;
    });
    return program.parse(argv);
  };

}).call(this);
