(function() {
  var File, Package, Source, debug, finished, info, util, _, _ref,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  util = require('./util');

  Source = require('./source').Source;

  File = require('./file').File;

  _ref = require('./command'), debug = _ref.debug, info = _ref.info, finished = _ref.finished;

  Package = (function(_super) {

    __extends(Package, _super);

    Package.types = function() {
      var type, _results;
      _results = [];
      for (type in this) {
        if (!__hasProp.call(this, type)) continue;
        if (type !== 'types' && type !== 'extend' && type !== 'create') {
          _results.push(type);
        }
      }
      return _results;
    };

    Package.extend = function() {
      var alias, packages, pkg, _i, _len, _results;
      packages = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      _results = [];
      for (_i = 0, _len = packages.length; _i < _len; _i++) {
        pkg = packages[_i];
        this[pkg.type] = pkg;
        _results.push((function() {
          var _j, _len1, _ref1, _ref2, _results1;
          _ref2 = (_ref1 = pkg.aliases) != null ? _ref1 : [];
          _results1 = [];
          for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
            alias = _ref2[_j];
            _results1.push(this[alias] = pkg);
          }
          return _results1;
        }).call(this));
      }
      return _results;
    };

    Package.create = function(options, sources, vendor) {
      var typ;
      if ((typ = this[options.type]) == null) {
        throw "Package type " + options.type + " not known";
      }
      return new typ(options, sources, vendor);
    };

    function Package(options, sources, vendorlibs) {
      var bundles, ctor, fs, lib, src, _i, _j, _len, _len1, _ref1, _ref2,
        _this = this;
      this.options = options;
      this.vendorlibs = vendorlibs;
      fs = require('fs');
      ctor = this.constructor;
      _.defaults(options, {
        compress: true,
        build: './build',
        bundles: []
      });
      _ref1 = this.options, this.name = _ref1.name, this.compress = _ref1.compress, this.build = _ref1.build, bundles = _ref1.bundles;
      this.bundlePaths = _.isString(bundles) ? JSON.parse(fs.readFileSync(bundles)) : bundles;
      this.files = {};
      this.sources = {};
      this._ready = false;
      this._pendingSources = 0;
      this.on('ready', function() {
        return _this._ready = true;
      });
      for (_i = 0, _len = sources.length; _i < _len; _i++) {
        src = sources[_i];
        this.registerSource(Source.create(src, this));
      }
      _ref2 = this.vendorlibs.libraries(ctor.type);
      for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
        lib = _ref2[_j];
        _.defaults(lib, {
          watch: false
        });
        this.registerSource(Source.create(lib, this));
      }
      this.on('newfile', function(file) {
        var bundle, compressed, fpath, type, _ref3;
        type = _this.constructor.type;
        if ((_ref3 = (fpath = file.relpath), __indexOf.call(_this.bundlePaths, _ref3) >= 0) && file.type === type) {
          bundle = _this.file(fpath, "" + type + "-bundle", _this.bundlePath(file));
          bundle.dependOnImports(file, _.bind(_this.bundle, _this));
          bundle.impermanent = true;
          bundle.parent = file;
          bundle.register();
          if (_this.compress) {
            compressed = _this.file(fpath, "" + type + "-bundle-minified", _this.compressedPath(file));
            compressed.dependOn(bundle, _.bind(_this.compressFile, _this));
            compressed.impermanent = true;
            return compressed.register();
          }
        }
      });
    }

    Package.prototype.file = function(relpath, type, fullpath, src) {
      var file, _base, _files, _ref1;
      if ((_ref1 = (_base = this.files)[type]) == null) {
        _base[type] = {};
      }
      file = (_files = this.files[type])[relpath];
      if (file == null) {
        file = _files[relpath] = new File(relpath, type, this);
      }
      if (fullpath != null) {
        file.attach(fullpath, src);
      }
      return file;
    };

    Package.prototype.registerSource = function(src) {
      var _base, _name, _ref1,
        _this = this;
      ((_ref1 = (_base = this.sources)[_name = src.constructor.type]) != null ? _ref1 : _base[_name] = []).push(src);
      this._pendingSources++;
      return src.files(null, function(files) {
        if (--_this._pendingSources === 0) {
          return _this.emit('ready');
        }
      });
    };

    Package.prototype.bundle = function(imported, bundle, cb) {
      var i, imports, iter, ws;
      i = 0;
      imports = bundle.parent.tsortedImports();
      ws = bundle.writeStream();
      ws.on('close', function() {
        finished('Packaged', bundle.fullpath);
        return cb();
      });
      iter = function() {
        var rs;
        if (i === imports.length) {
          ws.end();
          return;
        }
        rs = imports[i++].readStream();
        if (rs == null) {
          return iter();
        }
        rs.pipe(ws, {
          end: false
        });
        return rs.on('end', iter);
      };
      return iter();
    };

    Package.prototype.bundlePath = function(file) {
      return (require('path')).join(this.build, util.changeext(file.relpath, this.constructor.ext));
    };

    Package.prototype.compressedPath = function(file) {
      return (require('path')).join(this.build, this.compress === true ? util.changeext(file.relpath, this.constructor.compressedext) : _.template(compress)({
        filename: file.relpath
      }));
    };

    Package.prototype.requiredModules = function() {
      return _.chain(this.sources).values().flatten().invoke('requiredModules').flatten().uniq().value();
    };

    Package.prototype.ready = function(cb) {
      if (this._ready) {
        return cb();
      } else {
        return this.on('ready', cb);
      }
    };

    Package.prototype.actualize = function(cb) {
      var _this = this;
      return this.ready(function() {
        var allFiles, file, files, i, iter, relpath, roots, type, _ref1;
        allFiles = [];
        _ref1 = _this.files;
        for (type in _ref1) {
          files = _ref1[type];
          for (relpath in files) {
            file = files[relpath];
            allFiles.push(file);
          }
        }
        roots = _.filter(allFiles, function(file) {
          return file.liabilities.length === 0;
        });
        i = 0;
        iter = function() {
          if (i === roots.length) {
            return process.nextTick(cb);
          }
          return roots[i++].actualize(iter);
        };
        return iter();
      });
    };

    Package.prototype.watch = function(reset, ready) {
      var _this = this;
      return this.actualize(function() {
        var file, files, relpath, sources, src, type, _i, _len, _ref1, _ref2;
        _ref1 = _this.sources;
        for (type in _ref1) {
          sources = _ref1[type];
          for (_i = 0, _len = sources.length; _i < _len; _i++) {
            src = sources[_i];
            src.watch(reset);
          }
        }
        _ref2 = _this.files;
        for (type in _ref2) {
          files = _ref2[type];
          for (relpath in files) {
            file = files[relpath];
            file.watch(reset);
          }
        }
        if (ready != null) {
          return ready();
        }
      });
    };

    Package.prototype.unwatch = function() {
      var file, files, relpath, sources, src, type, _i, _len, _ref1, _ref2, _results;
      _ref1 = this.sources;
      for (type in _ref1) {
        sources = _ref1[type];
        for (_i = 0, _len = sources.length; _i < _len; _i++) {
          src = sources[_i];
          src.unwatch();
        }
      }
      _ref2 = this.files;
      _results = [];
      for (type in _ref2) {
        files = _ref2[type];
        _results.push((function() {
          var _results1;
          _results1 = [];
          for (relpath in files) {
            file = files[relpath];
            _results1.push(file.unwatch());
          }
          return _results1;
        })());
      }
      return _results;
    };

    Package.prototype.clean = function() {
      var _this = this;
      return this.ready(function() {
        var file, _i, _len, _ref1, _results;
        _ref1 = _this.impermanents();
        _results = [];
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          file = _ref1[_i];
          _results.push(file.unlinkSync());
        }
        return _results;
      });
    };

    Package.prototype.impermanents = function() {
      var acc, file, files, type, _i, _len, _ref1, _ref2;
      acc = [];
      _ref1 = this.files;
      for (type in _ref1) {
        files = _ref1[type];
        _ref2 = _.values(files);
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          file = _ref2[_i];
          if (file.impermanent === true) {
            acc.push(file);
          }
        }
      }
      return acc;
    };

    return Package;

  })((require('events')).EventEmitter);

  exports.Package = Package;

}).call(this);
